@@ -19,12 +19,12 @@
  *
  */
 
+#include "config.h"
 #include "cc-datetime-panel.h"
 
 #include <sys/time.h>
 #include "cc-timezone-map.h"
-#include "set-timezone.h"
-#include "libgnome-control-center/cc-lockbutton.h"
+#include "dtm.h"
 #include "date-endian.h"
 
 #include <gdesktop-enums.h>
@@ -78,6 +78,11 @@ struct _CcDateTimePanelPrivate
   GDesktopClockFormat clock_format;
 
   guint update_id;
+
+  DateTimeMechanism *dtm;
+  GCancellable *cancellable;
+
+  GPermission *permission;
 };
 
 static void update_time (CcDateTimePanel *self);
@@ -138,19 +143,49 @@ cc_date_time_panel_dispose (GObject *object)
       priv->date = NULL;
     }
 
+  if (priv->cancellable)
+    {
+      g_cancellable_cancel (priv->cancellable);
+      g_object_unref (priv->cancellable);
+      priv->cancellable = NULL;
+    }
+
+  if (priv->dtm)
+    {
+      g_object_unref (priv->dtm);
+      priv->dtm = NULL;
+    }
+
+  if (priv->permission)
+    {
+      g_object_unref (priv->permission);
+      priv->permission = NULL;
+    }
+
   G_OBJECT_CLASS (cc_date_time_panel_parent_class)->dispose (object);
 }
 
+static GPermission *
+cc_date_time_panel_get_permission (CcPanel *panel)
+{
+  CcDateTimePanelPrivate *priv = CC_DATE_TIME_PANEL (panel)->priv;
+
+  return priv->permission;
+}
+
 static void
 cc_date_time_panel_class_init (CcDateTimePanelClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  CcPanelClass *panel_class = CC_PANEL_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (CcDateTimePanelPrivate));
 
   object_class->get_property = cc_date_time_panel_get_property;
   object_class->set_property = cc_date_time_panel_set_property;
   object_class->dispose = cc_date_time_panel_dispose;
+
+  panel_class->get_permission = cc_date_time_panel_get_permission;
 }
 
 static void
@@ -253,13 +288,21 @@ update_time (CcDateTimePanel *self)
 }
 
 static void
-set_time_cb (CcDateTimePanel *self,
-             GError          *error)
+set_time_cb (GObject      *source,
+             GAsyncResult *res,
+             gpointer      user_data)
 {
-  /* TODO: display any error in a user friendly way */
-  if (error)
+  CcDateTimePanel *self = user_data;
+  GError *error;
+
+  error = NULL;
+  if (!date_time_mechanism_call_set_time_finish (self->priv->dtm,
+                                                 res,
+                                                 &error))
     {
+      /* TODO: display any error in a user friendly way */
       g_warning ("Could not set system time: %s", error->message);
+      g_error_free (error);
     }
   else
     {
@@ -268,24 +311,40 @@ set_time_cb (CcDateTimePanel *self,
 }
 
 static void
-set_timezone_cb (CcDateTimePanel *self,
-                 GError          *error)
+set_timezone_cb (GObject      *source,
+                 GAsyncResult *res,
+                 gpointer      user_data)
 {
-  /* TODO: display any error in a user friendly way */
-  if (error)
+  CcDateTimePanel *self = user_data;
+  GError *error;
+
+  error = NULL;
+  if (!date_time_mechanism_call_set_timezone_finish (self->priv->dtm,
+                                                     res,
+                                                     &error))
     {
+      /* TODO: display any error in a user friendly way */
       g_warning ("Could not set system timezone: %s", error->message);
+      g_error_free (error);
     }
 }
 
 static void
-set_using_ntp_cb (CcDateTimePanel *self,
-                  GError          *error)
+set_using_ntp_cb (GObject      *source,
+                  GAsyncResult *res,
+                  gpointer      user_data)
 {
-  /* TODO: display any error in a user friendly way */
-  if (error)
+  CcDateTimePanel *self = user_data;
+  GError *error;
+
+  error = NULL;
+  if (!date_time_mechanism_call_set_using_ntp_finish (self->priv->dtm,
+                                                      res,
+                                                      &error))
     {
+      /* TODO: display any error in a user friendly way */
       g_warning ("Could not set system to use NTP: %s", error->message);
+      g_error_free (error);
     }
 }
 
@@ -296,7 +355,12 @@ queue_set_datetime (CcDateTimePanel *self)
 
   /* for now just do it */
   unixtime = g_date_time_to_unix (self->priv->date);
-  set_system_time_async (unixtime, (GFunc) set_time_cb, self, NULL);
+
+  date_time_mechanism_call_set_time (self->priv->dtm,
+                                     unixtime,
+                                     self->priv->cancellable,
+                                     set_time_cb,
+                                     self);
 }
 
 static void
@@ -306,7 +370,12 @@ queue_set_ntp (CcDateTimePanel *self)
   gboolean using_ntp;
   /* for now just do it */
   using_ntp = gtk_switch_get_active (GTK_SWITCH (W("network_time_switch")));
-  set_using_ntp_async (using_ntp, (GFunc) set_using_ntp_cb, self, NULL);
+
+  date_time_mechanism_call_set_using_ntp (self->priv->dtm,
+                                          using_ntp,
+                                          self->priv->cancellable,
+                                          set_using_ntp_cb,
+                                          self);
 }
 
 static void
@@ -315,7 +384,11 @@ queue_set_timezone (CcDateTimePanel *self)
   /* for now just do it */
   if (self->priv->current_location)
     {
-      set_system_timezone_async (self->priv->current_location->zone, (GFunc) set_timezone_cb, self, NULL);
+      date_time_mechanism_call_set_timezone (self->priv->dtm,
+                                             self->priv->current_location->zone,
+                                             self->priv->cancellable,
+                                             set_timezone_cb,
+                                             self);
     }
 }
 
@@ -472,15 +545,20 @@ location_changed_cb (CcTimezoneMap   *map,
 }
 
 static void
-get_timezone_cb (CcDateTimePanel *self,
-                 const gchar     *timezone,
-                 GError          *error)
+get_timezone_cb (GObject      *source,
+                 GAsyncResult *res,
+                 gpointer      user_data)
 {
+  CcDateTimePanel *self = user_data;
   GtkWidget *widget;
+  gchar *timezone;
+  GError *error;
 
-  if (error)
+  error = NULL;
+  if (!date_time_mechanism_call_get_timezone_finish (self->priv->dtm, &timezone, res, &error))
     {
       g_warning ("Could not get current timezone: %s", error->message);
+      g_error_free (error);
     }
   else
     {
@@ -505,6 +583,8 @@ get_timezone_cb (CcDateTimePanel *self,
   g_signal_connect (self->priv->map, "location-changed",
                     G_CALLBACK (location_changed_cb), self);
 
+
+  g_free (timezone);
 }
 
 /* load region and city tree models */
@@ -754,15 +834,16 @@ on_permission_changed (GPermission *permission,
                        gpointer     data)
 {
   CcDateTimePanelPrivate *priv = CC_DATE_TIME_PANEL (data)->priv;
-  gboolean allowed;
+  gboolean allowed, using_ntp;
 
   allowed = g_permission_get_allowed (permission);
+  using_ntp = gtk_switch_get_active (GTK_SWITCH (W("network_time_switch")));
 
   /* All the widgets but the lock button and the 24h setting */
   gtk_widget_set_sensitive (W("map-vbox"), allowed);
   gtk_widget_set_sensitive (W("hbox2"), allowed);
   gtk_widget_set_sensitive (W("alignment2"), allowed);
-  gtk_widget_set_sensitive (W("table1"), allowed);
+  gtk_widget_set_sensitive (W("table1"), allowed && !using_ntp);
 }
 
 static void
@@ -814,13 +895,26 @@ cc_date_time_panel_init (CcDateTimePanel *self)
   GtkTreeModelSort *city_modelsort;
   guint i, num_days;
   gboolean using_ntp;
+  gboolean can_use_ntp;
   int ret;
-  GtkWidget *lockbutton;
-  GPermission *permission;
   DateEndianess endianess;
+  GError *error;
 
   priv = self->priv = DATE_TIME_PANEL_PRIVATE (self);
 
+  priv->cancellable = g_cancellable_new ();
+  error = NULL;
+  priv->dtm = date_time_mechanism_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                                          G_DBUS_PROXY_FLAGS_NONE,
+                                                          "org.gnome.SettingsDaemon.DateTimeMechanism",
+                                                          "/",
+                                                          priv->cancellable,
+                                                          &error);
+  if (priv->dtm == NULL) {
+        g_warning ("could not get proxy for DateTimeMechanism: %s", error->message);
+        g_error_free (error);
+  }
+
   priv->builder = gtk_builder_new ();
 
   ret = gtk_builder_add_objects_from_file (priv->builder, DATADIR"/datetime.ui",
@@ -835,7 +929,18 @@ cc_date_time_panel_init (CcDateTimePanel *self)
     }
 
   /* set up network time button */
-  using_ntp = get_using_ntp ();
+  error = NULL;
+  using_ntp = can_use_ntp = FALSE;
+  if (!date_time_mechanism_call_get_using_ntp_sync (priv->dtm,
+                                                    &can_use_ntp,
+                                                    &using_ntp,
+                                                    priv->cancellable,
+                                                    &error))
+    {
+      g_warning ("Failed to get using ntp: %s", error->message);
+      g_error_free (error);
+    }
+
   gtk_switch_set_active (GTK_SWITCH (W("network_time_switch")), using_ntp);
   update_widget_state_for_ntp (self, using_ntp);
   g_signal_connect (W("network_time_switch"), "notify::active",
@@ -925,28 +1030,25 @@ cc_date_time_panel_init (CcDateTimePanel *self)
 
   /* After the initial setup, so we can be sure that
    * the model is filled up */
-  get_system_timezone_async ((GetTimezoneFunc) get_timezone_cb, self, NULL);
+  date_time_mechanism_call_get_timezone (priv->dtm,
+                                         priv->cancellable,
+                                         get_timezone_cb,
+                                         self);
 
   queue_clock_update (self);
 
   /* add the lock button */
-  permission = polkit_permission_new_sync ("org.gnome.settingsdaemon.datetimemechanism.configure", NULL, NULL, NULL);
-  if (permission == NULL)
+  priv->permission = polkit_permission_new_sync ("org.gnome.settingsdaemon.datetimemechanism.configure", NULL, NULL, NULL);
+  if (priv->permission == NULL)
     {
       g_warning ("Your system does not have the '%s' PolicyKit files installed. Please check your installation",
                  "org.gnome.settingsdaemon.datetimemechanism.configure");
       return;
     }
 
-  /* DtLockButton takes ownership of the permission */
-  lockbutton = cc_lock_button_new (permission);
-  gtk_widget_set_margin_top (lockbutton, 12);
-  gtk_widget_show (lockbutton);
-  gtk_box_pack_end ((GtkBox *) gtk_builder_get_object (priv->builder, "hbox"),
-                    lockbutton, FALSE, FALSE, 0);
-  g_signal_connect (permission, "notify",
+  g_signal_connect (priv->permission, "notify",
                     G_CALLBACK (on_permission_changed), self);
-  on_permission_changed (permission, NULL, self);
+  on_permission_changed (priv->permission, NULL, self);
 }
 
 void
